import { extend, isFunction, join } from "lodash-es";
import { t as toSnakeObject, a as toCamleObject, s as setupMessage } from "./index-OSCkcmz2.js";
import { M, b, c, i, u, d } from "./index-OSCkcmz2.js";
import "@vue/runtime-core";
const defaultApiConfig = {
  baseUrl: "",
  requestTimeout: 6e4,
  successCode: 0,
  needLoginCode: [630],
  tenant: () => null,
  authToken: () => null,
  autoLogin: () => Promise.reject(new Error("autoLogin not implemented"))
};
let apiConfig = { ...defaultApiConfig };
let request = null;
const setupApi = (initer, config) => {
  if (config) {
    apiConfig = extend(apiConfig, config);
  }
  request = initer(apiConfig);
};
const useRequest = () => {
  if (!request) {
    throw new Error("Please call `setupApi()` before useRequest");
  }
  return request;
};
const fetch = ({ url, method, headers, params, data, contentType, responseType, timeout, onProgress, baseURL }) => {
  return useRequest()({
    baseURL,
    url,
    method: method || "get",
    params: toSnakeObject(params),
    data,
    responseType,
    headers: {
      ...headers,
      "Content-Type": contentType || "application/json"
    },
    timeout,
    onUploadProgress: onProgress,
    onDownloadProgress: onProgress
  }).then((res) => {
    if (res.status !== 200) {
      throw new Error(res.statusText);
    }
    return toCamleObject(res.data);
  });
};
const upload = (config) => {
  return fetch({
    url: config.url,
    baseURL: config.baseURL,
    method: config.method || "post",
    headers: config.headers,
    params: config.params,
    data: config.data,
    contentType: "multipart/form-data",
    responseType: config.responseType,
    timeout: config.timeout | 36e5,
    onProgress: config.onProgress
  });
};
const get = (config) => {
  return fetch({
    url: config.url,
    baseURL: config.baseURL,
    method: "get",
    headers: config.headers,
    params: config.params,
    responseType: config.responseType,
    timeout: config.timeout
  });
};
const post = (config) => {
  return fetch({
    url: config.url,
    baseURL: config.baseURL,
    method: "post",
    headers: config.headers,
    params: config.params,
    data: { body: toSnakeObject(config.data) },
    contentType: config.contentType,
    responseType: config.responseType,
    timeout: config.timeout
  });
};
const getPage = (config) => {
  return fetch({
    url: config.url,
    baseURL: config.baseURL,
    method: "get",
    headers: config.headers,
    params: extend(config.params, config.page, config.sort),
    responseType: config.responseType,
    timeout: config.timeout
  });
};
const postPage = (config) => {
  return fetch({
    url: config.url,
    baseURL: config.baseURL,
    method: "post",
    headers: config.headers,
    params: extend(config.params, config.page, config.sort),
    data: { body: toSnakeObject(config.data) },
    contentType: config.contentType,
    responseType: config.responseType,
    timeout: config.timeout
  });
};
const defineUploadApi = (config, defaultParam) => {
  return (data, param, onProgress) => {
    return upload({ ...config, params: extend({}, defaultParam, param), data, onProgress: onProgress ?? config.onProgress });
  };
};
const defineGetApi = (config, defaultParam) => {
  return (param) => {
    return get({ ...config, params: extend({}, defaultParam, param) });
  };
};
const defineGetPageApi = (config, defaultParam) => {
  return (params, page, sort) => {
    return getPage({ ...config, params: extend({}, defaultParam, params), page, sort });
  };
};
const definePostApi = (config, defaultParam) => {
  return (data, param) => {
    return post({ ...config, params: extend({}, defaultParam, param), data });
  };
};
const definePostPageApi = (config, defaultParam) => {
  return (data, page, param, sort) => {
    return postPage({ ...config, params: extend({}, defaultParam, param), data, page, sort });
  };
};
const defineCrudApi = (config, defaultParam) => {
  const getFn = isFunction(config.get) ? config.get : (id) => {
    return get({ url: `${config.path || ""}/${config.get || "id"}`, params: extend({}, defaultParam, { id }) });
  };
  const listFn = isFunction(config.list) ? config.list : (data, page, sort) => {
    return postPage({ url: `${config.path || ""}/${config.list || "list"}`, params: extend({}, defaultParam), data, page, sort });
  };
  const addFn = isFunction(config.add) ? config.add : (data) => {
    return post({ url: `${config.path || ""}/${config.add || "add"}`, params: extend({}, defaultParam), data });
  };
  const updateFn = isFunction(config.update) ? config.update : (id, data) => {
    return post({ url: `${config.path || ""}/${config.update || "update"}`, params: extend({}, defaultParam, { id }), data });
  };
  const changeStatusFn = isFunction(config.status) ? config.status : (ids, status) => {
    return post({ url: `${config.path || ""}/${config.status || "status"}`, params: extend({}, defaultParam, { ids: join(ids, ","), status }) });
  };
  const delsFn = isFunction(config.delete) ? config.delete : (ids) => {
    return get({ url: `${config.path || ""}/${config.delete || "delete"}`, params: extend({}, defaultParam, { ids: join(ids, ",") }) });
  };
  return {
    getOne: getFn,
    listPage: listFn,
    addOne: addFn,
    updateOne: updateFn,
    delete: delsFn,
    changeStatus: changeStatusFn
  };
};
const defineTreeApi = (path) => {
  const getTree = (parentId) => {
    return get({ url: `${path}/tree`, params: { parentId } });
  };
  const getPickerTree = (parentId) => {
    return get({ url: `${path}/picker-tree`, params: { parentId } });
  };
  const getSearch = (data, page) => {
    return postPage({ url: `${path}/search`, data, page });
  };
  return {
    getTree,
    getPickerTree,
    getSearch
  };
};
const nopMessageFun = () => {
  console.warn("Please use `setupDinoCore({msgHandlers:H})` or `provide(MessageSymbol)` first before call `useMessage()`");
};
const nopMessageHandler = {
  success: nopMessageFun,
  info: nopMessageFun,
  warning: nopMessageFun,
  error: nopMessageFun
};
const setupDinoCore = (config) => {
  setupMessage(config.message ?? nopMessageHandler);
  setupApi(config.apiRequest, config.api);
};
export {
  M as MessageSymbol,
  b as asArray,
  c as awaitGet,
  defaultApiConfig,
  defineCrudApi,
  defineGetApi,
  defineGetPageApi,
  definePostApi,
  definePostPageApi,
  defineTreeApi,
  defineUploadApi,
  fetch,
  get,
  getPage,
  i as isPromise,
  post,
  postPage,
  setupApi,
  setupDinoCore,
  setupMessage,
  toCamleObject,
  toSnakeObject,
  upload,
  u as useID,
  d as useMessage,
  useRequest
};
